/*
  WatchdogTest (EWB version)
  Verifies the proper operation of the EWB hardware watchdog timer

  This example code is in the public domain.
 */

#define eth_reset	(9)
#define wd_rfsh     (22)
#define wd_latch    (21)
#define Reset       (23)

unsigned long	startTimer;
unsigned long	curTimer;
unsigned		latch;
int				latchFailure	= false;
int				wdFailure 		= false;
int				testDone 		= false;


// Must send a positive pulse to the TA8030s
// to refresh the watchdog timer:

void watchdogRefresh( void )
{
    digitalWrite( wd_rfsh, 1 );
    delay( 1 );
    digitalWrite( wd_rfsh, 0 );
}

// Must send a negative-going pulse to the DS1834
// chip to cause a reset. The pulse must be at least
// 350 msec wide.

void toggleReset( void )
{
        digitalWrite( Reset, 0 );
        delay( 350 );
        digitalWrite( Reset, 1 );
}

// the setup routine runs once when you press reset:

void setup()
{

    Serial.begin( 9600 );

    // Stuff needed by the watchdog timer:

    pinMode( Reset, OUTPUT );       // Reset pin is an output.
    pinMode( wd_rfsh, OUTPUT );     // Watchdog refresh is an output.
    pinMode( wd_latch, INPUT );    // Read watchdog state here.

    delay( 1000 );
    Serial.println( "Watchdog test program" );

    watchdogRefresh();              // Guarantee that the WD is refreshed
    toggleReset();                  // Reset watchdog timer latch.
    delay(500);						// Give relay time to open, etc.
    toggleReset();                  // Reset watchdog timer latch.
    delay(500);						// Give relay time to open, etc.
    watchdogRefresh();              // Guarantee that the WD is refreshed
    delay( 100 );
    latch = !digitalRead( wd_latch );
    if( latch )
    {
    	Serial.println( "WD Latch failed to clear after reset/refresh" );
    	latchFailure = true;
    }
    startTimer = millis();			// Time at start of timeout period

}

// the loop routine runs over and over again forever:

void loop()
{
	curTimer = (millis() - startTimer);
	if(  curTimer == 6000 && digitalRead( wd_latch ) && !wdFailure)
	{
		Serial.println( "Watchdog timer failed to expire" );
		wdFailure = true;
	}
	if( curTimer == 6001 && !digitalRead( wd_latch ) )
	{
		toggleReset();
		delay( 350 );
		if( !digitalRead( wd_latch ))
		{
			Serial.println( "Failed to reset watchdog timer" );
		}
	}
	if( curTimer >= 10000 && !testDone )
	{
		if( !digitalRead( wd_latch ))
		{
			Serial.println( "Watchdog timer failed to expire, again" );
		}
		Serial.println( "Watchdog test complete" );
		testDone = true;
	}

}
